// Copyright 1996 by Jon Dart.  All Rights Reserved.

// Utility to add ECO codes to PGN game files.

#include "board.h"
#include "legal.h"
#include "movegen.h"
#include "options.h"
#include "movearr.h"
#include "notation.h"
#include "chessio.h"
#include "eco.h"
#include "movearr.h"
#include "strpair.h"
#include "bearing.h"
#include "globals.h"
#include "util.h"
#include "chessio.h"
#include "hashable.h"
#include "pool.h"

extern "C"
{
#include <stddef.h>
#include <string.h>
#include <ctype.h>
};
#include <fstream>
#include <iostream>
#include <ctype.h>
#include <map>
#include <vector>

using namespace std;

int replace = 1;
static int sample = 0;
static int max_ply = 40;
static const int MARGIN = 75;

static void show_usage()
{
	 cerr << "Usage: ecocoder pgn_file" << endl;
}



int CDECL main(int argc, char **argv)
{
   init_options(argv[0]);
   options.book.book_enabled = options.log_enabled = 0;
   init_globals(argv[0],0,0);
   atexit(cleanup_globals);
   
   Board board;
   ECO eco;
   int arg = 1;
   int minMoves = 30;
   int minELO = 0;

   if (argc <=1)
   {
         show_usage();
	 return -1;
   }
   else
   {
	ifstream pgn_file( argv[arg], ios::in | ios::binary);
	int c;
        ColorType side;
        string result, white, black;
	if (!pgn_file.good()) {
	  cerr << "could not open file " << argv[arg] << endl;
          exit(-1);
	}
        else
	{
            ArasanVector<StringPair> hdrs;
            while (!pgn_file.eof())
            {
               long first;
               MoveArray moves;
               board.Reset();
               side = White;
               while (pgn_file.good() && (c = pgn_file.get()) != EOF)
               {
                   if (c=='[')
                   {
                      pgn_file.putback(c);
                      break;
                   }
               }
               hdrs.removeAll();
               ChessIO::collect_headers(pgn_file,hdrs,first);
               //cout << " got " << hdrs.length() << " headers" << endl;
               for (;;)
               {
                   static char buf[2048];
                   static char num[20];
                   ChessIO::Token tok = ChessIO::get_next_token(pgn_file,buf,2048);
                   if (tok == ChessIO::Eof)
                      break;
                   if (tok == ChessIO::Number)
                   {
                      strncpy(num,buf,20);
                   }
                   else if (tok == ChessIO::GameMove)
                   {
                      // parse the move
                      Move m = Notation::Value(board,side,buf);
                      if (IsNull(m) ||
                        !legal_move(board,StartSquare(m),
                            DestSquare(m)))
                      {
                          // echo to both stdout and stderr
                          fprintf(stderr,"Illegal move: %s\n",buf);
                          cout << "Illegal move: " << buf << endl;
                          break;
                      }
                      else
                      {
			              Board_State bs(board);
                          char img[20];
						  // convert to SAN
                          Notation::Image(board,m,img);
                          moves.add_move(board,bs,m,img);
                          board.DoMove(m);
                      }
                      side = OppositeColor(side);
                  }
                  else if (tok == ChessIO::Unknown)
                  {
                      fprintf(stderr,"Unrecognized text: %s\n",buf);
                  }
                  else if (tok == ChessIO::Comment)
                  {
                      // ignored for now
                  }
                  else if (tok == ChessIO::Result)
                  {
                       result = buf;
                       break;
                  }
               }
               // output header
               string ecoC = "";
               int found = ChessIO::get_header(hdrs,"ECO",ecoC);
               if (!found) {
                  string name;
                  eco.classify(moves,ecoC,name);
                  if (ecoC != "") {
                     StringPair ecoHeader("ECO",ecoC.c_str());
                     hdrs.append(ecoHeader);
                  }
               }         
               else if (ecoC == "") {
				   string name;
                  eco.classify(moves,ecoC,name);
                  if (ecoC != "") {
                     StringPair ecoHeader("ECO",ecoC.c_str());
                     for (int i = 0; i < hdrs.length(); i++) {
                        const StringPair &hdr = hdrs[i];
                        if (hdr.tag == "ECO") { // replace old header
                           hdrs.set(i,ecoHeader);
                           break;
                        }
                     }
                  }
               }
               if (moves.length()>0) ChessIO::store_pgn(cout,moves,result,hdrs);
            }
            pgn_file.close();
        }
   }

   return 0;
}
